{ the following routines are written to show how simply read MAPI mail if
  you have Windows Messaging setup on your computer. This is in no way an
  e-mail program. It is just an example showing how to read and send mail
  using what Delphi provides. I did wrote this because I wanted to know
  how to write a simple e-mail program using MAPI. I found a great
  component by Danny Heijl, E-mail=Danny.Heijl@cevi.be,
  http://www.cevi.be/pp/danny/dhe.htm. I recommend this component if you
  are deciding to write a MAPI mail program.
  Most of the ideas are from Danny's code which I modified to make it in to
  simple MAPI calls. Thank you Danny for the great work.

  Feel free to add change delete or do whatever you please with this code.
  Drop me a line at tdang@softwarelabs.com if you feel the code is helpful.
}
unit reader;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls, mapi;

type
  TfrmReadMAPI = class(TForm)
    txtProfile: TEdit;
    lblProfile: TLabel;
    txtPPassword: TEdit;
    lblProfilePassword: TLabel;
    statMAPI: TStatusBar;
    btnUnread: TBitBtn;
    btnReadNext: TBitBtn;
    txtFrom: TEdit;
    txtTo: TEdit;
    moBody: TMemo;
    lblFrom: TLabel;
    txtSubject: TEdit;
    lblTo: TLabel;
    lblSubject: TLabel;
    lblMessageBody: TLabel;
    procedure btnUnreadClick(Sender: TObject);
    procedure btnReadNextClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    function GetCurrentUnReadID(oldID: integer): integer;
  public
    { Public declarations }
  end;

var
   frmReadMAPI: TfrmReadMAPI;
   GlobalUnreadCount: integer;
   hSession: ULONG;
   szMessageID: PChar;
   szSeedMessageID: PChar;
   flFlags: ULONG;
   lppMapiMessage: PMapiMessage;

const
  MsgIDSize = 520; { max allow - I guess }
  RECIP_MAX = MaxInt div SizeOf(TMapiRecipDesc);

implementation

{$R *.DFM}

procedure TfrmReadMAPI.btnUnreadClick(Sender: TObject);
var
   hSession        : ULONG;
   szMessageID     : PChar;
   szSeedMessageID : PChar;
   flFlags         : ULONG;
   Count           : ULONG;
begin
     { With Windows Messaging, I found that if you got the wrong
       Profile name, it will use the Windows default. My profile
       below did not have a password so I left it blank. }
     if MAPILogon(0,PChar(txtProfile.Text),PChar(txtPPassword.Text),0,0,@hSession) = SUCCESS_SUCCESS then
     begin
          Count := 0;
          statMAPI.SimpleText := '';
          { if successful login then allocate message id's }
          szMessageID := StrAlloc(MsgIDSize); { MAPI 32bit !! }
          szSeedMessageID := StrAlloc(MsgIDSize); { MAPI 32bit !! }
          FillChar(szMessageID^, MsgIDSize, 0);
          FillChar(szSeedMessageID^, MsgIDSize, 0);
          flFlags := MAPI_UNREAD_ONLY;

          inc(flFlags, MAPI_LONG_MSGID);
          { counting unread messages }
          while MapiFindNext( hSession, 0, nil, szSeedMessageID,
                        flFlags, 0, @szMessageID^) = SUCCESS_SUCCESS do
          begin
               if StrComp(szSeedMessageId, szMessageId) = 0 then
                  break
               else
                   StrCopy(szSeedMessageId,szMessageId);
               inc(Count);
          end;
          GlobalUnreadCount := Count;
          if Count > 0 then
             btnReadNext.Enabled := True;
          statMAPI.SimpleText := IntToStr(Count)+' unread messages';
          StrDispose(szMessageID);
          StrDispose(szSeedMessageID);
          MAPILogoff(hSession,0,0,0);
     end;
end;

procedure TfrmReadMAPI.btnReadNextClick(Sender: TObject);
const
   tmpStr: PChar = ':';
var
   AnyStr: PChar;
begin
     if MAPILogon(0,PChar(txtProfile.Text),PChar(txtPPassword.Text),0,0,@hSession) = SUCCESS_SUCCESS then
     begin
          { the following code is used to control what you want to see
            Danny lets you control by 1. having the ability to look at
            the message without marking it as read, 2. do/don't look at
            the attachment, and 3. Header only
          if FLeaveUnread   then flFlags := MAPI_PEEK;
          if FNOAttachments then flFlags := flFlags or MAPI_SUPPRESS_ATTACH;
          if FHeaderOnly    then flFlags := flFlags or MAPI_ENVELOPE_ONLY;
          }
          { I just want to read the mail for now and you can play around
            with attachment later }
          flFlags := MAPI_PEEK;
          flFlags := flFlags or MAPI_SUPPRESS_ATTACH;
          if GlobalUnreadCount <> 0 then
          begin
               szMessageId := StrAlloc(MsgIDSize);
               StrPCopy(szMessageId,IntToStr(GetCurrentUnReadID(-1)));
               if MapiReadMail(hSession, 0, szMessageId, flFlags,
                                0, lppMapiMessage) <> SUCCESS_SUCCESS then
               begin
                    MAPILogoff(hSession,0,0,0);
                    Exit;
               end
               else
               begin
                    { this does not list MAPI_CC, MAPI_BCC, or other things
                      in the header. Experiment with those yourself...}

                    { filtering out the "SMTP:" in the header
                      i.e. SMTP:tdang@softwarelabs.com }
                    if lppMapiMessage.lpOriginator.ulRecipClass = MAPI_ORIG then
                    begin
                         if lppMapiMessage.lpOriginator.lpszAddress <> '' then
                         begin
                              txtFrom.Text :=lppMapiMessage.lpOriginator.lpszAddress;
                              AnyStr := StrPos(PChar(txtFrom.Text),tmpStr);
                              if AnyStr <> nil then
                              begin
                                   Inc(AnyStr);
                                   txtFrom.Text := lppMapiMessage.lpOriginator.lpszName+'<'+AnyStr+'>';
                              end
                              else
                                  txtFrom.Text := lppMapiMessage.lpOriginator.lpszName+
                                    '<'+lppMapiMessage.lpOriginator.lpszAddress+'>';
                         end;
                    end;
                    if lppMapiMessage.lpRecips.ulRecipClass = MAPI_TO then
                    begin
                         if lppMapiMessage.lpRecips.lpszAddress <> '' then
                         begin
                              txtTo.Text := lppMapiMessage.lpRecips.lpszAddress;
                              AnyStr := StrPos(PChar(txtTo.Text),tmpStr);
                              if AnyStr <> nil then
                              begin
                                   Inc(AnyStr);
                                   txtTo.Text := AnyStr;
                              end
                              else
                                  txtTo.Text := lppMapiMessage.lpRecips.lpszAddress;
                         end;
                    end;
                    txtSubject.Text := lppMapiMessage.lpszSubject;
                    moBody.Lines.Text := lppMapiMessage.lpszNoteText;
                    Dec(GlobalUnreadCount);
                    statMAPI.SimpleText := IntToStr(GlobalUnreadCount)+' unread messages';
                        // this should be optional
                    if MAPIDeleteMail(hSession,0,szMessageID,0,0) <>
                           SUCCESS_SUCCESS then
                           MessageBox(Handle,'Cannot delete message','Warning',MB_OK+MB_ICONWARNING);
               end; // else starting reading mail
               StrDispose(szMessageId);
          end; // GlobalUnreadCount <> 0
          MAPILogoff(hSession,0,0,0);
     end; // mapi logon
end;

procedure TfrmReadMAPI.FormCreate(Sender: TObject);
begin
     szMessageId := StrAlloc(MsgIdSize);
     szSeedMessageID := StrAlloc(MsgIDSize);
     FillChar(szMessageID^, MsgIdSize, 0);
     FillChar(szSeedMessageID^, MsgIDSize, 0);
end;

procedure TfrmReadMAPI.FormDestroy(Sender: TObject);
begin
//     StrDispose(szMessageID);
//     StrDispose(szSeedMessageID);
end;

function TfrmReadMAPI.GetCurrentUnReadID(oldID: integer): integer;
var
   hSession        : ULONG;
   szMessageID     : PChar;
   szSeedMessageID : PChar;
   flFlags         : ULONG;
   NoMore          : Bool;
begin
     NoMore := True;
     Result := -1;
     { With Windows Messaging, I found that if you got the wrong
       Profile name, it will use the Windows default. My profile
       below did not have a password so I left it blank. }
     if MAPILogon(0,PChar(txtProfile.Text),PChar(txtPPassword.Text),0,0,@hSession) = SUCCESS_SUCCESS then
     begin
          statMAPI.SimpleText := '';
          { if successful login then allocate message id's }
          szMessageID := StrAlloc(MsgIDSize); { MAPI 32bit !! }
          szSeedMessageID := StrAlloc(MsgIDSize); { MAPI 32bit !! }
          FillChar(szMessageID^, MsgIDSize, 0);
//          FillChar(szSeedMessageID^, MsgIDSize, 0);
          StrPCopy(szSeedMessageID,IntToStr(oldID));
          flFlags := MAPI_UNREAD_ONLY;

          inc(flFlags, MAPI_LONG_MSGID);
          { counting unread messages }
          while MapiFindNext( hSession, 0, nil, szSeedMessageID,
                        flFlags, 0, @szMessageID^) = SUCCESS_SUCCESS do
//          if MapiFindNext( hSession, 0, nil, szSeedMessageID,
//                        flFlags, 0, @szMessageID^) = SUCCESS_SUCCESS then
          begin
               if StrComp(szSeedMessageId, szMessageId) = 0 then
                  break
               else
                   StrCopy(szSeedMessageId,szMessageId);
               NoMore := False;
               break;
          end;
          MAPILogoff(hSession,0,0,0);
          if not NoMore then
             Result := StrToInt(szMessageId)
          else
              Result := -1;
          StrDispose(szMessageID);
          StrDispose(szSeedMessageID);
     end;
end;

end.
